import numpy as np
from sklearn.pipeline import Pipeline
from skl_utils import *

if __name__ == '__main__':
    data = {
        'Age': [28, 34, 29, 42, 23, 19, 22, 30],
        # 'Size': ['M', 'S', 'S', 'L', 'M', 'L', 'S', 'S'],
        # 'Sex': ['M', 'M', 'M', 'F', 'M', 'M', 'F', 'F'],
        # 'Country': ['USA', 'China', 'USA', 'China', 'France', 'France', 'China', 'USA'],
        # 'Salary': [1000, 2500, 1200, 5000, 500, 250, None, 2400],
        # 'Num_Children': [2, 0, 0, 3, 2, 1, 4, 3],
        # 'Num_Pet': [5, 1, 0, 5, 2, 2, 3, 2]
    }
    df = pd.DataFrame(data)
    
    def inverse(X):
        return np.exp(X) - 1

    numeric_transformer = Pipeline([
        # ('select', ColumnExtractor(columns=['Age'])),
        # ('imputer', SimpleImputer(strategy='median')),
        # ('scaler', MinMaxScaler())
        ('select', FunctionTransformer(np.log1p, inverse)),
    ])
    
    print(df)

    data = numeric_transformer.fit_transform(df)

    print(data)
    
    reverse = numeric_transformer.inverse_transform(data)
    print(reverse)



from sklearn.pipeline import Pipeline
from skl_utils import *

if __name__ == '__main__':
    data = {
        'Age': [28, 34, 29, 42, 23, 19, 22, 30],
        'Size': ['M', 'S', 'S', 'L', 'M', 'L', 'S', 'S'],
        'Sex': ['M', 'M', 'M', 'F', 'M', 'M', 'F', 'F'],
        'Country': ['USA', 'China', 'USA', 'China', 'France', 'France', 'China', 'USA'],
        'Salary': [1000, 2500, 1200, 5000, 500, 250, None, 2400],
        'Num_Children': [2, 0, 0, 3, 2, 1, 4, 3],
        'Num_Pet': [5, 1, 0, 5, 2, 2, 3, 2]
    }
    df = pd.DataFrame(data)

    numeric_features = ['Age', 'Salary', 'Num_Children', 'Num_Pet']
    nominal_features = ['Sex', 'Country', 'Size']

    numeric_transformer = Pipeline([
        ('imputer', SimpleImputer(strategy='median')),
        ('scaler', MinMaxScaler())])

    nominal_transformer = Pipeline([
        ('imputer', SimpleImputer(strategy='most_frequent')),
        ('onehot', OneHotEncoder(drop='first'))])

    preprocessor = ColumnTransformer(
        transformers=[
            ('num', numeric_transformer, numeric_features),
            ('nom', nominal_transformer, nominal_features),
        ],
        remainder='passthrough')
    
    data = preprocessor.fit_transform(df)

    print(data)



import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
from functools import reduce
from sklearn import base
from sklearn import compose
from sklearn import preprocessing
from sklearn import impute


class ColumnExtractor(base.TransformerMixin):

    def __init__(self, dtype_include=None, dtype_exclude=None, columns=None):
        self.dtype_include = dtype_include
        self.dtype_exclude = dtype_exclude
        self.columns = columns

    def fit(self, X, y=None):
        return self

    def transform(self, X):
        if isinstance(X, pd.DataFrame):
            if self.dtype_include is not None or self.dtype_exclude is not None:
                return X.select_dtypes(include=self.dtype_include, exclude=self.dtype_exclude)
            else:
                return X[self.columns]
        else:
            return X


class ColumnSelector(compose.make_column_selector):

    def __init__(self, pattern=None, *, dtype_include=None, dtype_exclude=None, columns=None):
        if columns is not None:
            pattern = '|'.join(columns)
        super().__init__(pattern, dtype_include=dtype_include, dtype_exclude=dtype_exclude)


class FeatureUnion(base.TransformerMixin):

    def __init__(self, transformer_list):
        self.transformer_list = transformer_list

    def fit(self, X, y=None):
        for (name, t) in self.transformer_list:
            t.fit(X, y)
        return self

    def transform(self, X):
        Xts = [t.transform(X) for _, t in self.transformer_list]
        Xunion = reduce(lambda X1, X2: pd.merge(X1, X2, left_index=True, right_index=True), Xts)
        return Xunion


class FunctionTransformer(preprocessing.FunctionTransformer):

    def transform(self, X):
        Xf = super().transform(X)
        if isinstance(X, pd.DataFrame):
            Xf = pd.DataFrame(Xf, index=X.index, columns=X.columns)
        return Xf

    def inverse_transform(self, X):
        Xf = super().inverse_transform(X)
        if isinstance(X, pd.DataFrame):
            Xf = pd.DataFrame(Xf, index=X.index, columns=X.columns)
        return Xf


class OrdinalEncoder(preprocessing.OrdinalEncoder):

    def transform(self, X):
        Xf = super().transform(X)
        if isinstance(X, pd.DataFrame):
            Xf = pd.DataFrame(Xf, index=X.index, columns=X.columns)
        return Xf

    def inverse_transform(self, X):
        Xf = super().inverse_transform(X)
        if isinstance(X, pd.DataFrame):
            Xf = pd.DataFrame(Xf, index=X.index, columns=X.columns)
        return Xf


class OneHotEncoder(preprocessing.OneHotEncoder):

    def transform(self, X):
        Xf = super().transform(X)
        if isinstance(X, pd.DataFrame):
            Xf = pd.DataFrame(Xf.toarray(), index=X.index, columns=self.get_feature_names(X.columns))
        return Xf

    def inverse_transform(self, X):
        Xf = super().inverse_transform(X)
        if isinstance(X, pd.DataFrame):
            try:
                col_name = X.columns.tolist()[0].split('_')[0]
                Xf = pd.DataFrame(Xf, index=X.index, columns=[col_name])
            except RuntimeError:
                pass
        return Xf


class SimpleImputer(impute.SimpleImputer):

    def transform(self, X):
        Xf = super().transform(X)
        if isinstance(X, pd.DataFrame):
            Xf = pd.DataFrame(Xf, index=X.index, columns=X.columns)
        return Xf


class StandardScaler(preprocessing.StandardScaler):

    def transform(self, X, copy=None):
        Xf = super().transform(X, copy=None)
        if isinstance(X, pd.DataFrame):
            Xf = pd.DataFrame(Xf, index=X.index, columns=X.columns)
        return Xf

    def inverse_transform(self, X, copy=None):
        Xf = super().inverse_transform(X, copy=None)
        if isinstance(X, pd.DataFrame):
            Xf = pd.DataFrame(Xf, index=X.index, columns=X.columns)
        return Xf


class MinMaxScaler(preprocessing.MinMaxScaler):

    def transform(self, X):
        Xf = super().transform(X)
        if isinstance(X, pd.DataFrame):
            Xf = pd.DataFrame(Xf, index=X.index, columns=X.columns)
        return Xf

    def inverse_transform(self, X):
        Xf = super().inverse_transform(X)
        if isinstance(X, pd.DataFrame):
            Xf = pd.DataFrame(Xf, index=X.index, columns=X.columns)
        return Xf


class RobustScaler(preprocessing.RobustScaler):

    def transform(self, X):
        Xf = super().transform(X)
        if isinstance(X, pd.DataFrame):
            Xf = pd.DataFrame(Xf, index=X.index, columns=X.columns)
        return Xf

    def inverse_transform(self, X):
        Xf = super().inverse_transform(X)
        if isinstance(X, pd.DataFrame):
            Xf = pd.DataFrame(Xf, index=X.index, columns=X.columns)
        return Xf


class ColumnTransformer(compose.ColumnTransformer):

    def _hstack(self, Xs):
        try:
            # assumes Xs is DataFrame list
            if type(Xs) == list and all(isinstance(x, pd.DataFrame) for x in Xs):
                return pd.concat(Xs, axis=1)
            else:
                return super()._hstack(Xs)
        except RuntimeError:
            return super()._hstack(Xs)


def plot_decision_boundary(clf, x1, x2, y, labels=['0', '1'], cmap=plt.cm.Paired, figsize=(8, 7), step=0.1):
    # set figure size
    plt.figure(figsize=figsize)

    # generate mesh grid
    x_min, x_max = x1.min() - 20 * step, x1.max() + 20 * step
    y_min, y_max = x2.min() - 20 * step, x2.max() + 20 * step
    xx, yy = np.meshgrid(np.arange(x_min, x_max, step), np.arange(y_min, y_max, step))

    # predict labels in mesh grid
    z = clf.predict(np.c_[xx.ravel(), yy.ravel()]).reshape(xx.shape)

    # plot mesh grid
    plt.pcolormesh(xx, yy, z, cmap=cmap, alpha=.9)

    # plot data points
    scatter = plt.scatter(x1, x2, c=y, cmap=cmap, edgecolors='k', s=50, linewidths=1)
    plt.legend(handles=scatter.legend_elements()[0], labels=labels)

    return plt



from setuptools import setup, find_packages

def readme():
    with open('README.md') as f:
        return f.read()

def requirement():
    return [
        'matplotlib',
        'pandas',
        'scikit-learn'
    ]

setup(
    name='skl-utils',
    packages=find_packages(),
    version='0.0.3',
    description='Scikit-Learn Ultilities',
    long_description=readme(),
    long_description_content_type='text/markdown',
    author='LEANG Sotheara',
    author_email='leangsotheara@gmail.com',
    url='https://github.com/sotheara-leang/skl_utils',
    keywords=['scikit-learn', 'pandas', 'utilities'],
    install_requires=requirement(),
    python_requires=">=3.5",
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License'
    ]
)
